/*
 * $Id: wcfxsvar.h 101 2005-03-07 21:36:56Z riz $
 *
 */

#ifndef __WCFXSVAR_H__
#define __WCFXSVAR_H__

#ifdef _KERNEL

#define NUM_REGS	  109
#define NUM_INDIRECT_REGS 105

struct wcfxs_stats {
	int tipvolt;	/* TIP voltage (mV) */
	int ringvolt;	/* RING voltage (mV) */
	int batvolt;	/* VBAT voltage (mV) */
};

struct wcfxs_regs {
	unsigned char direct[NUM_REGS];
	unsigned short indirect[NUM_INDIRECT_REGS];
};

struct wcfxs_regop {
	int indirect;
	unsigned char reg;
	unsigned short val;
};

#define WCFXS_GET_STATS	_IOR (ZT_CODE, 60, struct wcfxs_stats)
#define WCFXS_GET_REGS	_IOR (ZT_CODE, 61, struct wcfxs_regs)
#define WCFXS_SET_REG	_IOW (ZT_CODE, 62, struct wcfxs_regop)

#define MINPEGTIME	10 * 8		/* 30 ms peak to peak gets us no more than 100 Hz */
#define PEGTIME		50 * 8		/* 50ms peak to peak gets us rings of 10 Hz or more */
#define PEGCOUNT	5		/* 5 cycles of pegging means RING */

#define	WCFXS_MAX_CARDS		4	/* number of interfaces per board */
#define AUDIO_RINGCHECK 1
#undef AUDIO_RINGCHECK
struct wcfxs_dma {
	uint32_t		*chunk;
	bus_dmamap_t		dma_map;
	bus_dma_segment_t	dma_seg;
	int			dma_nseg;
};

/*
 * struct wcfxs_softc holds data specific to a single board
 */
struct wcfxs_softc {
	struct device		sc_dv;		/* generic device */
	bus_space_tag_t		sc_iot;
	bus_space_handle_t	sc_ioh;
	bus_addr_t		sc_iob;
	bus_size_t		sc_iosize;

	void			*sc_ih;
	bus_dma_tag_t		sc_dmat;
	pci_chipset_tag_t	sc_pct;
	struct wcfxs_dma	sc_read;
	struct wcfxs_dma	sc_write;

	char			*sc_variety;
	struct zt_span		sc_span;
	int			sc_ios;         /* i/o direction of AUX lines on Tiger320 */
	int			sc_usecount;
	int			sc_intcount;	/* interrupts so far */
	int			sc_curcard;
	int			sc_cards;
	int			sc_cardmap;	/* Bitmap of cards present */
	int			sc_pos;
	int			sc_flags;
	int			sc_dead;
	lock_t			sc_lock;

	int			sc_modtype[WCFXS_MAX_CARDS];
	/* FXO vs. FXS module stuff */
	union {
	    struct {
#ifdef AUDIO_RINGCHECK
		unsigned int pegtimer[WCFXS_MAX_CARDS];
		int pegcount[WCFXS_MAX_CARDS];
		int peg[WCFXS_MAX_CARDS];
		int ring[WCFXS_MAX_CARDS];
#else
		int wasringing[WCFXS_MAX_CARDS];
#endif
		int ringdebounce[WCFXS_MAX_CARDS];
		int offhook[WCFXS_MAX_CARDS];
		int battdebounce[WCFXS_MAX_CARDS];
		int nobatttimer[WCFXS_MAX_CARDS];
		int battery[WCFXS_MAX_CARDS];
	    } fxo;
	    struct {
		int oldrxhook[WCFXS_MAX_CARDS];
		int debouncehook[WCFXS_MAX_CARDS];
		int lastrxhook[WCFXS_MAX_CARDS];
		int debounce[WCFXS_MAX_CARDS];
		int ohttimer[WCFXS_MAX_CARDS];
		/* IDLE changing hook state */
		int idletxhookstate[WCFXS_MAX_CARDS]; 
		int lasttxhook[WCFXS_MAX_CARDS];
		int palarms[WCFXS_MAX_CARDS];
	    } fxs;
	} sc_mod;

	struct zt_chan		sc_chans[WCFXS_MAX_CARDS];
};

/* descriptions */
struct wcfxs_desc {
	char		*name;
	unsigned long	flags;
};

/* identity table */
struct wcfxs_ident {
	pci_vendor_id_t		vendor;
	pci_product_id_t	product;
	pci_vendor_id_t		subvendor;
	const struct wcfxs_desc	*device_info;
};

#endif /* _KERNEL */
#endif /* __WCFXSVAR_H__ */
